from django import template
from django.template.loader import render_to_string


register = template.Library()


class BaseStyleguideNode(template.Node):

    def __init__(self, styleguide, reference, template_name, nodelist):
        self.styleguide = styleguide
        self.reference = reference
        self.template_name = template_name
        self.nodelist = nodelist

    def __repr__(self):
        return '<StyleguideBlockNode>'

    @classmethod
    def as_tag(cls, parser, token):
        bits = token.split_contents()[1:]

        if len(bits) < 2:
            raise template.TemplateSyntaxError("styleguideblock expected at "
                "least two arguments")

        elif len(bits) == 2:
            styleguide, reference = bits
            template_name = '"pykss/styleguideblock.html"'

        elif len(bits) >= 3 and bits[2] != 'using':
            raise template.TemplateSyntaxError("styleguideblock expected using "
                "as the third argument")

        elif len(bits) == 3:
            raise template.TemplateSyntaxError("styleguideblock expects a "
                "template name after 'using'")

        else:
            styleguide, reference, _using, template_name = bits

        return cls.dispatch(parser, styleguide, reference, template_name)

    @classmethod
    def dispatch(cls, parser, styleguide, reference, template_name):
        raise NotImplementedError

    def render(self, context):
        styleguide = self.styleguide.resolve(context)
        reference = self.reference.resolve(context)
        template_name = self.template_name.resolve(context)

        sections = [sec for ref, sec in styleguide.sections.iteritems()
                    if ref.startswith(reference)]

        if self.nodelist:
            example = self.nodelist.render(context).strip()
            for section in sections:
                section.add_example(example)

        output = []

        for section in sections:
            context.update({'section': section})
            html = render_to_string(template_name, context)
            output.append(html)
            context.pop()

        return ''.join(output)


class StyleguideBlockNode(BaseStyleguideNode):

    @classmethod
    def dispatch(cls, parser, styleguide, reference, template_name):
        nodelist = parser.parse(('endstyleguideblock',))
        parser.delete_first_token()
        return cls(
            styleguide=parser.compile_filter(styleguide),
            reference=parser.compile_filter(reference),
            template_name=parser.compile_filter(template_name),
            nodelist=nodelist,
        )


class RenderStyleguideNode(BaseStyleguideNode):

    @classmethod
    def dispatch(cls, parser, styleguide, reference, template_name):
        return cls(
            styleguide=parser.compile_filter(styleguide),
            reference=parser.compile_filter(reference),
            template_name=parser.compile_filter(template_name),
            nodelist=None,
        )


@register.tag
def styleguideblock(parser, token):
    """
    {% styleguideblock styleguide "1.1" %}
        <button class="$modifier_class">Example Button</button>
    {% endstyleguideblock %}

    {% styleguideblock styleguide "1.1" using "custom.html" %}
        <button class="$modifier_class">Example Button</button>
    {% endstyleguideblock %}

    """
    return StyleguideBlockNode.as_tag(parser, token)


@register.tag
def renderstyleguide(parser, token):
    """
    {% renderstyleguide styleguide "1.1" %}

    {% renderstyleguide styleguide "1.1" using "custom.html" %}
    """
    return RenderStyleguideNode.as_tag(parser, token)



import os

from django.conf import settings
from django.template import Context, Template, TemplateSyntaxError
from django.test import TestCase
from django.utils import simplejson

from mock import patch, ANY

from pykss.parser import Parser


class StyleguideBlockTestCase(TestCase):

    def setUp(self):
        css = os.path.join(settings.PROJECT_ROOT, 'tests', 'fixtures', 'css')
        self.styleguide = Parser(css)

    def test_when_section_does_not_exist(self):
        template = Template("""
            {% load pykss %}
            {% styleguideblock styleguide "99" %}
            {% endstyleguideblock %}
        """)
        context = Context({'styleguide': self.styleguide})
        self.assertEquals(template.render(context).strip(), '')

    @patch('pykss.contrib.django.templatetags.pykss.render_to_string')
    def test_uses_default_template(self, mock_render_to_string):
        mock_render_to_string.return_value = ''
        template = Template("""
            {% load pykss %}
            {% styleguideblock styleguide "2.1.1" %}
            {% endstyleguideblock %}
        """)
        context = Context({'styleguide': self.styleguide})
        template.render(context)
        mock_render_to_string.assert_called_with('pykss/styleguideblock.html', ANY)

    @patch('pykss.contrib.django.templatetags.pykss.render_to_string')
    def test_allows_overiding_template(self, mock_render_to_string):
        mock_render_to_string.return_value = ''
        template = Template("""
            {% load pykss %}
            {% styleguideblock styleguide "2.1.1" using "custom.html" %}
            {% endstyleguideblock %}
        """)
        context = Context({'styleguide': self.styleguide})
        template.render(context)
        mock_render_to_string.assert_called_with('custom.html', ANY)

    def test_when_using_argument_is_wrong(self):
        text = """
            {% load pykss %}
            {% styleguideblock styleguide "2.1.1" "custom.html" %}
            {% endstyleguideblock %}
        """
        self.assertRaises(TemplateSyntaxError, Template, text)

    def test_when_using_is_defined_without_template(self):
        text = """
            {% load pykss %}
            {% styleguideblock styleguide "2.1.1" using %}
            {% endstyleguideblock %}
        """
        self.assertRaises(TemplateSyntaxError, Template, text)

    def test_renders_correctly(self):
        template = Template("""
            {% load pykss %}
            {% styleguideblock styleguide "2.1.1" using "django.html" %}
                <i class="main$modifier_class"></i>
            {% endstyleguideblock %}
        """)
        context = Context({'styleguide': self.styleguide})
        results = simplejson.loads(template.render(context))
        self.assertEqual(results['section'], '2.1.1')
        self.assertEqual(results['filename'], 'buttons.css')
        self.assertEqual(results['description'], 'Your standard form button.')
        self.assertEqual(results['modifiers'][0], [':hover', 'Highlights when hovering.'])
        self.assertEqual(results['example_html'], '<i class="main"></i>')
        self.assertEqual(results['modifier_examples'][0], [':hover', '<i class="main pseudo-class-hover"></i>'])


#class RenderStyleguideTestCase(TestCase):

#    def test_renders_correctly(self):



import os

from django.conf import settings
from django.test import TestCase

from pykss.contrib.django.views import StyleguideMixin, StyleguideView


class StyleguideMixinTestCase(TestCase):

    def test_get_styleguide_get_dirs_from_settings(self):
        css = os.path.join(settings.PROJECT_ROOT, 'tests', 'fixtures', 'css')
        with self.settings(PYKSS_DIRS=[css]):
            styleguide = StyleguideMixin().get_styleguide()
            self.assertEqual(styleguide.section('2.1.1').description, 'Your standard form button.')

    def test_get_context_data_adds_sytleguide_to_context(self):
        self.assertIn('styleguide', StyleguideView().get_context_data())



import os

from pykss.comment import CommentParser
from pykss.exceptions import SectionDoesNotExist
from pykss.section import Section


class Parser(object):

    def __init__(self, *paths):
        self.paths = paths

    def parse(self):
        sections = {}

        filenames = [os.path.join(subpath, filename)
            for path in self.paths
            for subpath, dirs, files in os.walk(path)
            for filename in files]

        for filename in filenames:
            parser = CommentParser(filename)
            for block in parser.blocks:
                section = Section(block, os.path.basename(filename))
                if section.section:
                    sections[section.section] = section

        return sections

    @property
    def sections(self):
        if not hasattr(self, '_sections'):
            self._sections = self.parse()
        return self._sections

    def section(self, reference):
        try:
            return self.sections[reference]
        except KeyError:
            raise SectionDoesNotExist('Section "%s" does not exist.' % reference)



import re

from pykss.modifier import Modifier


CLASS_MODIFIER = '.'
PSEUDO_CLASS_MODIFIER = ':'
MODIFIER_DESCRIPTION_SEPARATOR = ' - '
EXAMPLE_START = 'Example:'
REFERENCE_START = 'Styleguide'

reference_re = re.compile(r'%s ([\d\.]+)' % REFERENCE_START)
optional_re = re.compile(r'\[(.*)\]\?')


class Section(object):

    def __init__(self, comment=None, filename=None):
        self.comment = comment or ''
        self.filename = filename

    def parse(self):
        self._description_lines = []
        self._modifiers = []
        self._example_lines = []
        self._reference = None

        in_example = False

        for line in self.comment.splitlines():
            if line.startswith(CLASS_MODIFIER) or line.startswith(PSEUDO_CLASS_MODIFIER):
                try:
                    modifier, description = line.split(MODIFIER_DESCRIPTION_SEPARATOR)
                except ValueError:
                    pass
                else:
                    self._modifiers.append(Modifier(modifier.strip(), description.strip()))

            elif line.startswith(EXAMPLE_START):
                in_example = True

            elif line.startswith(REFERENCE_START):
                in_example = False
                self._reference = reference_re.match(line).groups()[0].rstrip('.')

            elif in_example is True:
                self._example_lines.append(line)

            else:
                self._description_lines.append(line)

        self._description = '\n'.join(self._description_lines).strip()
        self.add_example('\n'.join(self._example_lines).strip())

    @property
    def description(self):
        if not hasattr(self, '_description'):
            self.parse()
        return self._description

    @property
    def modifiers(self):
        if not hasattr(self, '_modifiers'):
            self.parse()
        return self._modifiers

    @property
    def example(self):
        if not hasattr(self, '_modifiers'):
            self.parse()
        return self._example

    @property
    def section(self):
        if not hasattr(self, '_reference'):
            self.parse()
        return self._reference

    def add_example(self, example):
        self._example = optional_re.sub('', example).replace('$modifier_class', '')
        for modifier in self._modifiers:
            modifier.add_example(optional_re.sub(r'\1', example))



import re


single_line_re = re.compile(r'^\s*\/\/')
single_line_strip_re = re.compile(r'\s*\/\/')

multi_line_start_re = re.compile(r'^\s*\/\*')
multi_line_end_re = re.compile(r'.*\*\/')
multi_line_start_strip_re = re.compile(r'\s*\/\*')
multi_line_end_strip_re = re.compile(r'\*\/')
multi_line_middle_strip_re = re.compile(r'^(\s*\*+)')

preceding_white_space_re = re.compile(r'^\s*')


def is_single_line_comment(line):
    return single_line_re.match(line) is not None


def is_multi_line_comment_start(line):
    return multi_line_start_re.match(line) is not None


def is_multi_line_comment_end(line):
    if is_single_line_comment(line):
        return False
    return multi_line_end_re.match(line) is not None


def parse_single_line(line):
    return single_line_strip_re.sub('', line).rstrip()


def parse_multi_line(line):
    cleaned = multi_line_start_strip_re.sub('', line)
    return multi_line_end_strip_re.sub('', cleaned).rstrip()


def normalize(lines):
    cleaned = []
    indents = []

    for line in lines:
        line = multi_line_middle_strip_re.sub('', line)
        cleaned.append(line)
        match = preceding_white_space_re.match(line)
        if line:
            indents.append(len(match.group()))

    indent = min(indents)

    return '\n'.join([line[indent:] for line in cleaned]).strip()


class CommentParser(object):

    def __init__(self, filename):
        self.filename = filename

    def parse(self):
        blocks = []
        current_block = []
        inside_single_line_block = False
        inside_multi_line_block = False

        with open(self.filename) as fileobj:
            for line in fileobj:
                # Parse single-line style
                if is_single_line_comment(line):
                    parsed = parse_single_line(line)

                    if inside_single_line_block:
                        current_block.append(parsed)
                    else:
                        current_block = [parsed]
                        inside_single_line_block = True

                # Prase multi-line style
                if is_multi_line_comment_start(line) or inside_multi_line_block:
                    parsed = parse_multi_line(line)

                    if inside_multi_line_block:
                        current_block.append(parsed)
                    else:
                        current_block = [parsed]
                        inside_multi_line_block = True

                # End a multi-line block if detected
                if is_multi_line_comment_end(line):
                    inside_multi_line_block = False

                # Store the current block if we're done
                if is_single_line_comment(line) is False and inside_multi_line_block is False:
                    if current_block:
                        blocks.append(normalize(current_block))

                    inside_single_line_block = False
                    current_block = []

        return blocks

    @property
    def blocks(self):
        if not hasattr(self, '_blocks'):
            self._blocks = self.parse()
        return self._blocks



from django.conf import settings
from django.views.generic.base import TemplateView

import pykss


class StyleguideMixin(object):

    def get_styleguide(self):
        dirs = getattr(settings, 'PYKSS_DIRS', [])
        return pykss.Parser(*dirs)

    def get_context_data(self, **kwargs):
        context = super(StyleguideMixin, self).get_context_data(**kwargs)
        context.update({'styleguide': self.get_styleguide()})
        return context


class StyleguideView(StyleguideMixin, TemplateView):
    pass



class Modifier(object):

    def __init__(self, name, description):
        self.name = name
        self.description = description
        self.example = ''

    @property
    def class_name(self):
        return self.name.replace('.', ' ').replace(':', ' pseudo-class-').strip()

    def add_example(self, example):
        self.example = example.replace('$modifier_class', ' %s' % self.class_name)
